@@ -14,11 +14,11 @@ Games::Cards::ShuffleTrack - Track cards through shuffles and cuts
 
 =head1 VERSION
 
-Version 0.01_5
+Version 0.01_6
 
 =cut
 
-our $VERSION = '0.01_5';
+our $VERSION = '0.01_6';
 
 my $cut_limits = {
 	normal  => [0.19, 0.82], # on a 52 cards deck, cut between 10 and 43 cards
@@ -40,10 +40,10 @@ my $decks = {
 };
 
 my $shortcuts = {
-	'top'		=> 1,
-	'second'	=> 2,
-	'greek'		=> -2,
-	'bottom'	=> -1,
+    'top'     => 1,
+    'second'  => 2,
+    'greek'   => -2,
+    'bottom'  => -1,
 };
 
 
@@ -61,7 +61,7 @@ This module allows you to simulate true and false shuffles and cuts.
 	$deck->riffle_shuffle();
 	print $deck->get_deck();
 
-Or perhaps with more control:
+Or perhaps with more precision:
 
 	my $deck = Games::Cards::ShuffleTrack->new();
 
@@ -147,22 +147,38 @@ sub new {
 	}, $self;
 }
 
-=head3 reorder
+=head3 restart
 
 Reset the deck to its original status. The original is whatever you selected when you created the deck.
 
-	$deck->reorder;
+	$deck->restart;
 
 Do bear in mind that by doing so you are replinishing the deck of whatever cards you took out of it.
 
 =cut
 
-sub reorder {
+sub restart {
 	my $self = shift;
 	return $self->_set_deck( @{$self->{'original'}} );
 }
 
 
+=head3 deck_size
+
+Returns the size of the deck.
+
+    for ( 1 .. $deck->deck_size ) {
+        ...
+    }
+
+=cut
+
+sub deck_size {
+	my $self = shift;
+	return scalar @{$self->{'deck'}};
+}
+
+
 =head3 get_deck
 
 Returns the deck (a reference to a list of strings).
@@ -309,7 +325,7 @@ If no number is given then no cards are run.
 
 sub run {
 	my $self            = shift;
-	my $number_of_cards = shift;
+	my $number_of_cards = shift || return $self;
 	my $where_to_drop   = shift || 'drop-bottom';
 
 	$number_of_cards > 0 or return $self;
@@ -357,7 +373,7 @@ sub riffle_shuffle {
 	my $depth = shift;
 
 	# cut the deck (left pile is the original top half)
-	my $cut_depth = _cut_depth( $self->_deck_size, $depth );
+	my $cut_depth = _cut_depth( $self->deck_size, $depth );
 
 	my @left_pile  = @{$self->get_deck};
 	my @right_pile = splice @left_pile, $cut_depth;
@@ -418,7 +434,7 @@ sub faro {
 
 	# TODO: what happens when the deck is odd-sized?
 	my @first_half  = @{$self->get_deck};
-	my @second_half = splice @first_half, $self->_deck_size / 2;
+	my @second_half = splice @first_half, $self->deck_size / 2;
 
 	$self->_set_deck(
 			$faro eq 'in' ?
@@ -482,12 +498,14 @@ sub cut {
 	my $self     = shift;
 	my $position = shift;
 
-	if (defined $position and abs($position) > $self->_deck_size) {
-		warn "Tried to cut the deck at a non-existing position ($position).\n";
-		return $self;
+	if (     defined $position
+		 and looks_like_number( $position )
+		 and abs($position) > $self->deck_size ) {
+				warn "Tried to cut the deck at a non-existing position ($position).\n";
+				return $self;
 	}
 
-	my $cut_depth = _cut_depth( $self->_deck_size, $position );
+	my $cut_depth = _cut_depth( $self->deck_size, $position );
 
 	my @deck = @{$self->get_deck};
 	unshift @deck, splice @deck, $cut_depth;
@@ -733,7 +751,9 @@ sub remove {
 
 	my @deck = @{$self->get_deck};
 
-	my $card = splice @deck, $position - 1, 1;
+    if ($position > 0) { $position--; }
+
+	my $card = splice @deck, $position, 1;
 
 	$self->_set_deck( @deck );
 	return $card;
@@ -792,7 +812,9 @@ sub take_random {
 	my $lower_limit = shift || 1;
 	my $upper_limit = shift;
 
-	$upper_limit = min( $upper_limit, $self->_deck_size );
+	$upper_limit = defined $upper_limit ?
+					$upper_limit :
+					$self->deck_size;
 
 	return $self->remove( _rand( $lower_limit, $upper_limit ) );
 }
@@ -805,11 +827,6 @@ sub _set_deck {
 	return $self->{'deck'} = [@_];
 }
 
-sub _deck_size {
-	my $self = shift;
-	return scalar @{$self->{'deck'}};
-}
-
 sub _rand {
 	my ($lower_limit, $upper_limit) = @_;
 
